<?php
namespace app\cxinv\model;

use think\facade\Log;use think\Model;
class TradePool extends Base
{
//设置字段信息
    protected $schema = [
        'id'  =>'bigint',//
        'tradNo'  =>'varchar',//资金编号
        'logNo'  =>'varchar',//资金交易编号
        'platform_type'  =>'tinyint',//0无属性 1ToB 2ToC
        'customerNo'  =>'varchar',//客户公司
        'companyNo'  =>'varchar',//业务公司编号
        'trade_time'  =>'datetime',//交易时间
        'total_fee'  =>'decimal',//总额
        'remark'  =>'varchar',//备注
        'tag_type'  =>'int',//资金类型 0 认领自己 1平台衍生资金认领
        'relaNo'  =>'varchar',//关联平台衍生资金编号
        'is_del'  =>'int',//
        'status'  =>'varchar',//状态 1待审核2审核通过3 驳回
        'apply_name'  =>'varchar',//
        'apply_id'  =>'int',//
        'addtime'  =>'datetime',//
        'updatetime'  =>'datetime',//
       ];
    protected $createTime='addtime';
    protected $updateTime='updatetime';
    
    public function Trade(){
        return $this->belongsTo(Trade::class,"tradNo","tradNo")
        ->bind(["trade_out","trade_in","trade_bank","companyName","pcompanyNo"=>"companyNo","customerName","pcustomerNo"=>"customerNo","ptotal_fee"=>"total_fee","trade_type","balance","used_fee","poCode","NdCode"]);
    }
    public function Lib(){
        return $this->belongsTo(TradeLib::class,"tradNo","tradNo")->bind(['palt_po','skuCode','goodName','goodNum','platPrice','plat_total','tax','cat_code','cat_name','comPrice','com_total','pcCode']);
    }

    public static function onAfterInsert(Model $model) : void{
        if($model->status==2 && $model->tag_type==0){
             self::makeChangeOrder($model->logNo,2);
        }
    }
    public static function onAfterUpdate(Model $model) : void{
        $changedData = $model->getChangedData();
        if(isset($changedData['status'])&& $model->tag_type==0){
            self::makeChangeOrder($model->logNo,$changedData['status']);
        }
    }
        //修改衍生数据

    public static function makeChangeOrder($logNo,$status){
        $info = self::with(["Trade","Lib"])->where("logNo",$logNo)->findOrEmpty();
        Log::info("修改衍生数据".json_encode($info));
        if($info->isEmpty()) throw new \Exception("数据不存在");
        if($info->trade_type==0) return;
        switch ($status){
            case 2:
               self::createTrade($info);
               break;
            case 4:
                self::returnTrade($info);
               break;
            default:
               throw new \Exception("状态错误");
        }

    }

    public static function createTrade($info){
          $assoc = Assoc::alias('a')
          ->leftJoin('qrd_info b','a.orderCode = b.sequenceNo')
          ->where('a.viceCode',$info->logNo)
          ->where(['a.status'=>2,'a.is_del'=>0,"a.type"=>2])
          ->where('b.cxCode','<>','')
          ->field('a.cancel_fee,a.order_total,b.sequenceNo,b.cxCode')
          ->select();
        if($assoc->isEmpty()) throw new \Exception('数据不存在');
        $qrdArr = QrdInfo::where(['cxCode'=>array_column($assoc->toArray(),'cxCode'),'is_comon'=>0,'is_del'=>0])
        ->column('totalPrice,wpay_fee,apay_fee,platform_type,companyNo,customerNo,sequenceNo,id','cxCode');
        $Asoc=[];
        $log=[];
        Log::info("订单数据".json_encode($qrdArr,JSON_UNESCAPED_UNICODE));
        foreach ($assoc->toArray() as $k=>$v){
            $orderLv= bcdiv($v['cancel_fee'],$v['order_total'],8);
            if(!isset($qrdArr[$v['cxCode']])) throw new \Exception("{$v['sequenceNo']}关联通用订单数据不存在");
             $trade_fee=round(bcmul($qrdArr[$v['cxCode']]['totalPrice'],$orderLv,8),2);
             $temp=['logNo' => makeNo('TRC',str_pad($k+1,4,'0',STR_PAD_LEFT)),
                     'tradNo' => $info->tradNo,
                     'platform_type' => $qrdArr[$v['cxCode']]['platform_type'],
                     'companyNo' => $qrdArr[$v['cxCode']]['companyNo'],
                     'customerNo' => $qrdArr[$v['cxCode']]['customerNo'],
                     'apply_id' =>0,
                     'apply_name' => 'system',
                     'trade_time' =>  $info->trade_time,
                     'total_fee' => $trade_fee,
                     "tag_type"=>1,//1平台衍生
                     'relaNo' => $info->logNo,
                     'status' => 2,//2审核通过
                     ];
             $Asoc[]=[
                'assocNo' =>makeNo('AS',str_pad($k+1,4,'0',STR_PAD_LEFT)),
                'apply_id' =>0,
                'apply_name' =>'system',
                'type' => 2,
                'orderCode' => $qrdArr[$v['cxCode']]['sequenceNo'],
                'customerNo' => $qrdArr[$v['cxCode']]['customerNo'],
                'viceCode' => $temp['logNo'],
                'order_total' =>$qrdArr[$v['cxCode']]['totalPrice'],
                'vice_total' => $info->plat_total,
                'cancel_fee' =>$trade_fee,
                'status' => 2,//认领通过
             ];
             $log[]=$temp;
             if($qrdArr[$v['cxCode']]['wpay_fee']<$trade_fee) throw new \Exception("{$qrdArr[$v['cxCode']]['sequenceNo']}待支付支付金额不足");
             $qrdArr[$v['cxCode']]['wpay_fee']-= $trade_fee;
             $qrdArr[$v['cxCode']]['apay_fee']+= $trade_fee;
             $qrdArr[$v['cxCode']]['pay_status']= ($qrdArr[$v['cxCode']]['wpay_fee']==0 &&$qrdArr[$v['cxCode']]['pay_fee']==0) ?3:2;
             $qrdArr[$v['cxCode']]['status']=1;
        }
           $qrdArr = array_values($qrdArr);
           $up= (new QrdInfo)->saveAll($qrdArr);
           if(!$up) throw new \Exception('通用订单数据修改失败');
           $up= (new Assoc)->saveAll($Asoc);
           if(!$up) throw new \Exception('关联数据创建失败');
           $up= (new TradePool)->saveAll($log);
           if(!$up) throw new \Exception('衍生数据创建失败');
    }

    public static function returnTrade($info){
        $return = TradeReturn::where(["logNo"=>$info->logNo,"status"=>1])->findOrEmpty();
        if($return->isEmpty()) throw new \Exception("{$info->logNo}退款记录未找到");
        $trade = self::where(["relaNo"=>$info->logNo,"tag_type"=>1,"status"=>2,"tradNo"=>$info->tradNo,"is_del"=>0])->select();
        if($trade->isEmpty()) throw new \Exception("未找到关联资金数据");
        $assoc = Assoc::alias('a')
          ->leftJoin('qrd_info b','a.orderCode = b.sequenceNo')
          ->whereIn('a.viceCode',array_column($trade->toArray(),"logNo"))
          ->where(['a.status'=>2,'a.is_del'=>0,'a.type'=>2])
          ->where('b.cxCode','<>','')
          ->field('a.cancel_fee,a.orderCode,a.viceCode,a.id,a.status')
          ->select();
        if($assoc->isEmpty()) throw new \Exception('关联通用订单数据不存在');
        $qrdArr = QrdInfo::where(['sequenceNo'=>array_column($assoc->toArray(),'orderCode'),'is_comon'=>0,'is_del'=>0])
                ->column('totalPrice,wpay_fee,pay_fee,apay_fee,inv_status,platform_type,companyNo,customerNo,sequenceNo,id','sequenceNo');
        if(empty($qrdArr)) throw new \Exception('通用订单数据不存在');
        foreach ($assoc->toArray() as $k=>&$v){
            if(!isset($qrdArr[$v['orderCode']])) throw new \Exception("{$v['sequenceNo']}关联通用订单数据不存在");
            Assoc::CheckTrad($v->viceCode,4);
        }
        $returns=[];
        foreach ($trade as $k=>$v){
            $v->status = 4;
           $up= $v->save();
           if(!$up) throw new \Exception('资金数据修改失败');
            $returns[]=[ 'returnCode'=>makeNo('RTA',str_pad($k+1,4,'0',STR_PAD_LEFT)),
            'logNo'=>$v->logNo,
            'companyNo'=>$v->companyNo,
            'tradNo'=>$v->tradNo,
            'return_img'=>'',
            'type'=>2,
            'apply_id'=>"0",
            'apply_name'=>"system",
            'return_reason'=>$return->return_reason,
            'remark'=>$return->remark,
            'status'=>1];
        }

        $up= (new TradeReturn)->saveAll($returns);
        if(!$up) throw new \Exception('退款记录创建失败');
    }

}